module Rex
  module Server
    # Implementation of a simple one node server setup
    #
    # Setups/Starts
    #
    # - a message processor to parse and forward
    #   messages to the matching engine
    # - a matching engine instance to process any order book related
    #   messages
    # - a websocket server to accept websocket base connections and
    #   process incoming messages
    class MultiMatchingEngineServerSetup
      def initialize(host, port)
        matching_engine_inbox_pipe = EventMachine::Channel.new

        matching_engine_out_channels = Hash[(1..3).map { |i| [i, EventMachine::Channel.new] }]

        @matching_engines = matching_engine_out_channels.map do |id,channel|
          MatchingEngine.new(
            matching_engine_inbox_pipe,
            channel,
            id: id
          )
        end

        matching_engine_published_channel = EventMachine::Channel.new
        @publisher = Publisher.new(matching_engine_out_channels.values, matching_engine_published_channel)


        message_processor_inflow_pipe = EventMachine::Channel.new
        message_broker = MessageBroker.new(JsonMessageSerializer.new)

        @message_processor = Rex::Server::MessageProcessor.new(
          matching_engine_inbox_pipe,
          matching_engine_published_channel,
          message_broker,
          message_processor_inflow_pipe
        )

        @websocket_server = Rex::Server::WebsocketServer.new(
          message_broker,
          message_processor_inflow_pipe,
          host: host,
          port: port
        )
      end

      def start
        @message_processor.start
        @publisher.start
        @matching_engines.each(&:start)
        @websocket_server.start
      end

      def stop
        @matching_engines.each(&:stop)
        @message_processor.stop
        @websocket_server.stop
      end
    end
  end
end
